@@ -65,7 +65,7 @@ def test_simplification_methods():
     line.remove_zero_length_drifts(inplace=True)
 
     # Test merging of multipoles
-    line._var_management = None
+    line.env._var_management = None
     line.insert_element(element=xt.Multipole(knl=[1, 0, 3], ksl=[0, 20, 0]), name='m1', at_s=3.3)
     line.insert_element(element=xt.Multipole(knl=[4, 2], ksl=[10, 40]), name='m2', at_s=3.3)
     line.insert_element(element=xt.Multipole(knl=[0, 3, 8], ksl=[2, 0, 17]), name='m3', at_s=3.3)
@@ -512,7 +512,7 @@ def test_from_dict_current():
     assert isinstance(d1, xt.Drift)
     assert d1.length == 4
 
-    assert d2 is d1
+    assert d2._obj is d1._obj  # they are views
 
     assert line.metadata == test_dict['metadata']
 
@@ -668,11 +668,11 @@ def test_optimize_multipoles(test_context):
 
     for nn in test_line.element_names:
         if nn in ('d1', 'd2'):
-            assert type(test_line.element_dict[nn]) is xt.SimpleThinBend
+            assert type(test_line.get(nn)) is xt.SimpleThinBend
         elif nn == 'q1' or nn == 'q3':
-            assert type(test_line.element_dict[nn]) is xt.SimpleThinQuadrupole
+            assert type(test_line.get(nn)) is xt.SimpleThinQuadrupole
         else:
-            assert type(test_line.element_dict[nn]) is xt.Multipole
+            assert type(test_line.get(nn)) is xt.Multipole
 
 def test_from_json_to_json(tmp_path):
 
@@ -691,7 +691,7 @@ def test_from_json_to_json(tmp_path):
     line.metadata = example_metadata
 
     def asserts():
-        assert len(result.element_dict.keys()) == 2
+        assert len(result.env.elements) == 2
         assert result.element_names == ['m', 'd', 'm', 'd']
 
         assert isinstance(result['m'], xt.Multipole)
@@ -1050,6 +1050,8 @@ def test_insert_thick_element_reuse_marker_name():
                 element_names=list(elements.keys()))
 
     # Note that the name is reused
+    line.remove('m1') # remove from the line
+    line.env.remove('m1') # remove from the environment
     line.insert_element(element=xt.Bend(length=1.), name='m1', at_s=0.5)
 
     tt = line.get_table()
