@@ -110,12 +110,12 @@ def _generate_track_local_particle_with_transformations(
             element_shape = 'curved'
             misalign_arguments = 'shift_x, shift_y, shift_s, rot_y_rad, rot_x_rad, rot_s_rad_no_frame, anchor, length, angle, rot_s_rad'
             get_angle = f'double const angle = {element_name}Data_get{add_to_call}_angle(el)'
-            get_rot_s_rad = 'double const rot_s_rad = atan2(_sin_rot_s, _cos_rot_s)'
         else:
             element_shape = 'straight'
-            misalign_arguments = 'shift_x, shift_y, shift_s, rot_y_rad, rot_x_rad, rot_s_rad_no_frame, anchor, length'
+            misalign_arguments = 'shift_x, shift_y, shift_s, rot_y_rad, rot_x_rad, rot_s_rad_no_frame, anchor, length, rot_s_rad'
             get_angle = ''
             get_rot_s_rad = ''
+        get_rot_s_rad = 'double const rot_s_rad = atan2(_sin_rot_s, _cos_rot_s)'
 
         if 'isthick' in xofields:
             get_length = (
@@ -143,6 +143,7 @@ def _generate_track_local_particle_with_transformations(
             f'    {get_length};\n'
             f'    {get_angle};\n'
             f'    double const anchor = {element_name}Data_get{add_to_call}_rot_shift_anchor(el);\n'
+            f'    int8_t const backtrack = LocalParticle_check_track_flag(part0, XS_FLAG_BACKTRACK);\n'
             '\n')
 
         if rot_and_shift_from_parent:
@@ -164,11 +165,11 @@ def _generate_track_local_particle_with_transformations(
             )
 
         source += (
-            f'    track_misalignment_entry_{element_shape}(part0, {misalign_arguments});'
-            '\n'
-            '     //start_per_particle_block (part0->part)\n'
-            '          SRotation_single_particle(part, _sin_rot_s, _cos_rot_s);\n'
-            '     //end_per_particle_block\n'
+            '     if (!backtrack) {\n'
+            f'      track_misalignment_entry_{element_shape}(part0, {misalign_arguments}, backtrack);'
+            '     } else {\n'
+            f'      track_misalignment_exit_{element_shape}(part0, {misalign_arguments}, backtrack);\n'
+            '     }\n'
             '\n'
             '    /* Spin tracking is disabled by the synrad compile flag */\n'
             '    #ifndef XTRACK_MULTIPOLE_NO_SYNRAD\n'
@@ -200,6 +201,7 @@ def _generate_track_local_particle_with_transformations(
             f'    {get_length};\n'
             f'    {get_angle};\n'
             f'    double const anchor = {element_name}Data_get{add_to_call}_rot_shift_anchor(el);\n'
+            f'    int8_t const backtrack = LocalParticle_check_track_flag(part0, XS_FLAG_BACKTRACK);\n'
             '\n'
             '    /* Spin tracking is disabled by the synrad compile flag */\n'
             '    #ifndef XTRACK_MULTIPOLE_NO_SYNRAD\n'
@@ -216,11 +218,11 @@ def _generate_track_local_particle_with_transformations(
             '       //end_per_particle_block\n'
             '    #endif\n'
             '\n'
-            '    //start_per_particle_block (part0->part)\n'
-            '       SRotation_single_particle(part, -_sin_rot_s, _cos_rot_s);\n'
-            '    //end_per_particle_block\n'
-            '\n'
-            f'    track_misalignment_exit_{element_shape}(part0, {misalign_arguments});'
+            '     if (!backtrack) {\n'
+            f'       track_misalignment_exit_{element_shape}(part0, {misalign_arguments}, backtrack);'
+            '     } else {\n'
+            f'       track_misalignment_entry_{element_shape}(part0, {misalign_arguments}, backtrack);\n'
+            '     }\n'
             '}\n'
         )
     else:
@@ -333,6 +335,7 @@ def _tranformations_active(self):
 def _disable_transformations_if_needed(self):
     if not _tranformations_active(self):
         self._sin_rot_s = -999.
+        self._cos_rot_s = -999.
     elif self._sin_rot_s < -2.:
         self._sin_rot_s = 0.
         self._cos_rot_s = 1.
@@ -744,6 +747,10 @@ class BeamElement(xo.HybridClass, metaclass=MetaBeamElement):
         dct = xo.HybridClass.to_dict(self, **kwargs)
         if self.name_associated_aperture is not None:
             dct['name_associated_aperture'] = self.name_associated_aperture
+        if hasattr(self, 'extra') and self.extra:
+            dct['extra'] = self.extra.copy()
+        if hasattr(self, 'prototype'):
+            dct['prototype'] = self.prototype
         return dct
 
     @classmethod
@@ -755,6 +762,11 @@ class BeamElement(xo.HybridClass, metaclass=MetaBeamElement):
 
         instance = xo.HybridClass._static_from_dict(cls, dct, **kwargs)
         instance.name_associated_aperture = name_associated_aperture
+
+        if 'extra' in dct.keys():
+            instance.extra = dct['extra'].copy()
+        if 'prototype' in dct.keys():
+            instance.prototype = dct['prototype']
         return instance
 
     def copy(self, **kwargs):
@@ -795,8 +807,8 @@ class Replica:
         return Replica(parent_name=self.parent_name)
 
     def resolve(self, element_container, get_name=False):
-        if hasattr(element_container, 'element_dict'):
-            element_container = element_container.element_dict
+        if hasattr(element_container, '_element_dict'):
+            element_container = element_container._element_dict
         target_name = self.parent_name
         visited = {target_name}
         while isinstance(element := element_container[target_name], Replica):
