@@ -6,25 +6,31 @@
 #ifndef XTRACK_SYNRAD_SPECTRUM_H
 #define XTRACK_SYNRAD_SPECTRUM_H
 
+#include <headers/track.h>
+#include <random/random_src/uniform_accurate.h>
+
+
 #define SQRT3 1.732050807568877
 #define ALPHA_EM 0.0072973525693
 
-/*gpufun*/
-void synrad_average_kick(LocalParticle* part, double curv, double lpath,
+#if defined(XTRACK_SYNRAD_SCALE_SAME_AS_FIRST) && !defined(XO_CONTEXT_CPU_SERIAL)
+#error "XTRACK_SYNRAD_SCALE_SAME_AS_FIRST not supported when multithreading"
+#endif
+
+GPUFUN
+void synrad_average_kick(LocalParticle* part, double B_T, double lpath,
                          double* dp_record, double* dpx_record, double* dpy_record
                         ){
     double const gamma0  = LocalParticle_get_gamma0(part);
     double const mass0 = LocalParticle_get_mass0(part);
     double const q0 = LocalParticle_get_q0(part);
 
-    double const Q0_coulomb = q0 * QELEM;
+    double const Q0_coulomb = fabs(q0) * QELEM;
     double const mass0_kg = mass0 / C_LIGHT / C_LIGHT * QELEM;
 
     double const delta  = LocalParticle_get_delta(part);
     double const gamma = gamma0 * (1 + delta); // Ultra-relativistic approximation
 
-    double const P0_J = LocalParticle_get_p0c(part) / C_LIGHT * QELEM;
-    double const B_T = curv * P0_J / (Q0_coulomb);
     double const r0_m = Q0_coulomb * Q0_coulomb/  (4 * PI * EPSILON_0 * mass0_kg * C_LIGHT * C_LIGHT);
 
     double const Ps_W = 2 * r0_m * C_LIGHT * Q0_coulomb * Q0_coulomb * gamma * gamma * B_T * B_T / (3 * mass0_kg);
@@ -40,36 +46,36 @@ void synrad_average_kick(LocalParticle* part, double curv, double lpath,
     }
     #endif
 
-    #ifdef XTRACK_SYNRAD_KICK_SAME_AS_FIRST
-    if (part -> ipart == 0){
-      *dp_record = LocalParticle_get_delta(part);
-      *dpx_record = LocalParticle_get_px(part);
-      *dpy_record = LocalParticle_get_py(part);
+    if (LocalParticle_check_track_flag(part, XS_FLAG_SR_KICK_SAME_AS_FIRST)){
+      if (part -> ipart == 0){
+        *dp_record = LocalParticle_get_delta(part);
+        *dpx_record = LocalParticle_get_px(part);
+        *dpy_record = LocalParticle_get_py(part);
+      }
+      else{
+        f_t = 1.0;
+      }
     }
-    else{
-      f_t = 1.0;
-    }
-    #endif
 
     LocalParticle_update_delta(part, (delta+1) * f_t - 1);
     LocalParticle_scale_px(part, f_t);
     LocalParticle_scale_py(part, f_t);
 
-    #ifdef XTRACK_SYNRAD_KICK_SAME_AS_FIRST
-    if (part -> ipart == 0){
-      *dp_record = LocalParticle_get_delta(part) - *dp_record;
-      *dpx_record = LocalParticle_get_px(part) - *dpx_record;
-      *dpy_record = LocalParticle_get_py(part) - *dpy_record;
-    }
+    if (LocalParticle_check_track_flag(part, XS_FLAG_SR_KICK_SAME_AS_FIRST)){
+      if (part -> ipart == 0){
+        *dp_record = LocalParticle_get_delta(part) - *dp_record;
+        *dpx_record = LocalParticle_get_px(part) - *dpx_record;
+        *dpy_record = LocalParticle_get_py(part) - *dpy_record;
+      }
     else{
       LocalParticle_update_delta(part, LocalParticle_get_delta(part) + *dp_record);
       LocalParticle_add_to_px(part, *dpx_record);
       LocalParticle_add_to_py(part, *dpy_record);
     }
-    #endif
+  }
 }
 
-/*gpufun*/
+GPUFUN
 double SynRad(double x)
 {
   // x :    energy normalized to the critical energy
@@ -167,7 +173,7 @@ double SynRad(double x)
   return synrad;
 }
 
-/*gpufun*/
+GPUFUN
 double synrad_gen_photon_energy_normalized(LocalParticle *part)
 {
   // initialize constants used in the approximate expressions
@@ -178,7 +184,9 @@ double synrad_gen_photon_energy_normalized(LocalParticle *part)
   double const a2 = 1.770750801624037; // Synrad(xlow)/exp(-xlow);
   double const c1 = 0.; //
   double const ratio = 0.908250405131381;
-  double appr, exact, result;
+  double appr=0;
+  double exact=1000;
+  double result=0;
   do {
     if (RandomUniform_generate(part) < ratio) { // use low energy approximation
       result=c1+(1.-c1)*RandomUniform_generate(part);
@@ -187,7 +195,9 @@ double synrad_gen_photon_energy_normalized(LocalParticle *part)
       exact=SynRad(result);
       appr=a1/tmp;
     } else {				// use high energy approximation
-      result=xlow-log(RandomUniform_generate(part));
+      double const u = RandomUniformAccurate_generate(part);
+      if (u < 1.e-50) continue;  // avoid log(0), retrigger generation
+      result=xlow-log(u);
       exact=SynRad(result);
       appr=a2*exp(-result);
     }
@@ -195,39 +205,50 @@ double synrad_gen_photon_energy_normalized(LocalParticle *part)
   return result; // result now exact spectrum with unity weight
 }
 
-/*gpufun*/
-double synrad_average_number_of_photons(
-                          double beta0_gamma0, double curv, double lpath){
+GPUFUN
+double synrad_average_number_of_photons(double mass0, double q0,
+                          double beta0_gamma0, double B_T, double lpath){
+
+    double const mass0_kg = mass0 * QELEM / C_LIGHT / C_LIGHT;
+    /*a*/ double const P0_J = mass0_kg * beta0_gamma0 * C_LIGHT;
+    double const Q0_coulomb = fabs(q0) * QELEM;
+    double const curv = B_T / P0_J * Q0_coulomb;
     double const kick = curv * lpath;
     return 2.5/SQRT3*ALPHA_EM*beta0_gamma0*fabs(kick);
 }
 
-/*gpufun*/
-int64_t synrad_emit_photons(LocalParticle *part, double curv /* 1/m */,
+GPUFUN
+int64_t synrad_emit_photons(LocalParticle *part, double B_T,
                             double lpath /* m */,
                             RecordIndex record_index,
                             SynchrotronRadiationRecordData record
                             ){
 
-    if (fabs(curv) < 1e-15)
+    if (fabs(B_T) < 1e-4)
         return 0;
 
     int64_t nphot = 0;
 
     // TODO Introduce effect of chi and mass_ratio!!!
-    double const m0 = LocalParticle_get_mass0(part); // eV
+    double const mass0 = LocalParticle_get_mass0(part); // eV
+    double const q0 = LocalParticle_get_q0(part); // e
     double const gamma0  = LocalParticle_get_gamma0(part);
     double const beta0  = LocalParticle_get_beta0(part);
 
+    double const Q0_coulomb = fabs(q0) * QELEM;
+
+    double const mass0_kg = mass0 * QELEM / C_LIGHT / C_LIGHT;
+    double const P0_J = mass0_kg * beta0 * gamma0 * C_LIGHT;
+    double const curv = B_T / P0_J * Q0_coulomb;
+
+    double const delta  = LocalParticle_get_delta(part);
+    double gamma = gamma0 * (1 + delta); // Ultra-relativistic approximation
+
     double const initial_energy = LocalParticle_get_energy0(part)
 	                          + LocalParticle_get_ptau(part)*LocalParticle_get_p0c(part); // eV
-    double energy = initial_energy;
-    double gamma = energy / m0; //
-    //double beta_gamma = sqrt(gamma*gamma-1); //
+    double energy = initial_energy; // eV
     double n = RandomExponential_generate(part); // path_length / mean_free_path;
-    // printf("curv = %e\n gamma0 = %e\n lpath = %e\n", curv, gamma0, lpath);
-    // printf("Average_number_of_photons = %e\n", synrad_average_number_of_photons(beta0 * gamma0, curv, lpath));
-    while (n < synrad_average_number_of_photons(beta0 * gamma0, curv, lpath)) {
+    while (n < synrad_average_number_of_photons(mass0, q0, beta0 * gamma0, B_T, lpath)) {
         nphot++;
         double const c1 = 1.5 * 1.973269804593025e-07; // hbar * c = 1.973269804593025e-07 eV * m
         double const energy_critical = c1 * (gamma*gamma*gamma0) * curv; // eV
@@ -237,7 +258,6 @@ int64_t synrad_emit_photons(LocalParticle *part, double curv /* 1/m */,
             break;
         }
         energy -= energy_loss; // eV
-        gamma = energy / m0; //
         // beta_gamma = sqrt(gamma*gamma-1); // that's how beta gamma is
         n += RandomExponential_generate(part);
         if (record){
