@@ -3,4 +3,4 @@
 # Copyright (c) CERN, 2021.                 #
 # ######################################### #
 
-from .loss_location_refinement import LossLocationRefinement
+from .loss_location_refinement import LossLocationRefinement, _skip_in_loss_location_refinement
