@@ -126,7 +126,7 @@ class MultiSetter(xo.HybridClass):
         line: xtrack.Line
             Line in which the elements are mutated
         elements: list of int or strings
-            List of indeces or names of the elements to be mutated.
+            List of indices or names of the elements to be mutated.
         field: str
             Name of the field to be mutated.
         index: int or None
@@ -163,7 +163,7 @@ class MultiSetter(xo.HybridClass):
         else:
             inner_obj = el
             inner_name = field
-        dd = getattr(inner_obj.copy(_context=xo.context_default), inner_name)
+        dd = getattr(inner_obj.copy(_context=xo.context_default)._xobject, inner_name)
         if index is not None:
             dd = dd[index]
         self.dtype = type(dd)
@@ -257,7 +257,7 @@ def _extract_offset(obj, field_name, index, dtype, xodtype):
         inner_name = field_name
 
     if index is None:
-        assert isinstance(getattr(inner_obj, inner_name), dtype), (
+        assert isinstance(getattr(inner_obj._xobject, inner_name), dtype), (
             "Inconsistent types")
         return inner_obj._xobject._get_offset(inner_name)
     else:
