@@ -119,6 +119,7 @@ class Particles(xo.HybridClass):
 
     def __init__(
             self,
+            pdg_id_0=None,
             _capacity=None,
             _no_reorganize=False,
             **kwargs,
@@ -136,6 +137,8 @@ class Particles(xo.HybridClass):
 
         Parameters
         ----------
+        pdg_id_0 : int or str, optional, define reference mass and charge from
+            PDG id or particle name.
         _capacity: int
             The maximum number of particles that can be stored in the object.
             If not provided, it is inferred from the size of the provided
@@ -176,6 +179,10 @@ class Particles(xo.HybridClass):
             Reference relativistic gamma
         beta0 : array_like of float, optional
             Reference relativistic beta
+        rigidity0 : array_like of float, optional
+            Reference magnetic rigidity [T.m]
+        kinetic_energy0 : array_like of float, optional
+            Reference kinetic energy [eV]
         mass_ratio : array_like of float, optional
             mass/mass0 (this is used to track particles of
             different species. Note that mass is the rest mass
@@ -221,15 +228,20 @@ class Particles(xo.HybridClass):
 
         accepted_args = set(self._xofields.keys()) | {
             'energy0', 'tau', 'pzeta', 'mass_ratio', 'mass', 'kinetic_energy0',
-            '_context', '_buffer', '_offset', 'p0', 'name',
+            '_context', '_buffer', '_offset', 'name', 'rigidity0',
         }
         if set(kwargs.keys()) - accepted_args:
             raise NameError(f'Invalid argument(s) provided: '
                             f'{set(kwargs.keys()) - accepted_args}')
 
+        if pdg_id_0 is not None:
+            _update_kwargs0_from_pdg_id(pdg_id_0, kwargs)
+
         per_part_input_vars = (
             self.per_particle_vars +
             ((xo.Float64, 'energy0'),
+             (xo.Float64, 'kinetic_energy0'),
+             (xo.Float64, 'rigidity0'),
              (xo.Float64, 'tau'),
              (xo.Float64, 'pzeta'),
              (xo.Float64, 'mass_ratio'))
@@ -316,11 +328,6 @@ class Particles(xo.HybridClass):
         self.start_tracking_at_element = kwargs.get(
                             'start_tracking_at_element', -1)
 
-        # Init refs
-        if 'kinetic_energy0' in kwargs.keys():
-            assert kwargs.get('energy0') is None
-            kwargs['energy0'] = kwargs.pop('kinetic_energy0') + self.mass0
-
         # Ensure that all per particle inputs are numpy arrays of the same
         # length, and move them to the target context
         for xotype, field in per_part_input_vars:
@@ -345,15 +352,6 @@ class Particles(xo.HybridClass):
         # Init independent per particle vars
         self.init_independent_per_part_vars(kwargs)
 
-
-        self._update_refs(
-            p0c=kwargs.get('p0c'),
-            energy0=kwargs.get('energy0'),
-            gamma0=kwargs.get('gamma0'),
-            beta0=kwargs.get('beta0'),
-            mask=input_mask,
-        )
-
         # Init chi and charge ratio
         self._update_chi_charge_ratio(
             chi=kwargs.get('chi'),
@@ -362,6 +360,17 @@ class Particles(xo.HybridClass):
             mask=input_mask,
         )
 
+        # Init reference momentum and related vars
+        self._update_refs(
+            p0c=kwargs.get('p0c'),
+            energy0=kwargs.get('energy0'),
+            gamma0=kwargs.get('gamma0'),
+            beta0=kwargs.get('beta0'),
+            kinetic_energy0=kwargs.get('kinetic_energy0'),
+            rigidity0=kwargs.get('rigidity0'),
+            mask=input_mask,
+        )
+
         # Init energy deviations
         self._update_energy_deviations(
             delta=kwargs.get('delta'),
@@ -1229,6 +1238,26 @@ class Particles(xo.HybridClass):
     def p0c(self, value):
         self.p0c[:] = value
 
+    def _rigidity0_setitem(self, indx, val):
+        ctx = self._buffer.context
+        temp_rigidity0 = ctx.zeros(shape=self._p0c.shape, dtype=np.float64)
+        temp_rigidity0[:] = np.nan
+        temp_rigidity0[indx] = val
+        self.update_p0c(temp_rigidity0 * clight * self.q0)
+
+    @property
+    def rigidity0(self):
+        rigidity0 = self.p0c / clight / self.q0
+        return self._buffer.context.linked_array_type.from_array(
+            rigidity0,
+            mode='setitem_from_container',
+            container=self,
+            container_setitem_name='_rigidity0_setitem')
+
+    @rigidity0.setter
+    def rigidity0(self, value):
+        self.rigidity0[:] = value
+
     def update_gamma0(self, new_gamma0):
 
         """
@@ -1263,6 +1292,7 @@ class Particles(xo.HybridClass):
     def gamma0(self, value):
         self.gamma0[:] = value
 
+
     def update_beta0(self, new_beta0):
 
         """
@@ -1309,19 +1339,48 @@ class Particles(xo.HybridClass):
             self._rpp, mode='readonly',
             container=self)
 
+    def _energy0_setitem(self, indx, val):
+        ctx = self._buffer.context
+        temp_gamma0 = ctx.zeros(shape=self._gamma0.shape, dtype=np.float64)
+        temp_gamma0[:] = np.nan
+        temp_gamma0[indx] = val / self.mass0
+        self.update_gamma0(temp_gamma0)
+
     @property
     def energy0(self):
         energy0 = (self.p0c * self.p0c + self.mass0 * self.mass0) ** 0.5
-        return self._buffer.context.linked_array_type.from_array(
-            energy0, mode='readonly',
-            container=self)
+        out  = self._buffer.context.linked_array_type.from_array(
+            energy0,
+            mode='setitem_from_container',
+            container=self,
+            container_setitem_name='_energy0_setitem')
+        return out
+
+    @energy0.setter
+    def energy0(self, value):
+        self.energy0[:] = value
+
+
+    def _kinetic_energy0_setitem(self, indx, val):
+        ctx = self._buffer.context
+        temp_gamma0 = ctx.zeros(shape=self._gamma0.shape, dtype=np.float64)
+        temp_gamma0[:] = np.nan
+        temp_gamma0[indx] = val / self.mass0 + 1
+        self.update_gamma0(temp_gamma0)
 
     @property
     def kinetic_energy0(self):
         kene0 = self.energy0 - self.mass0
-        return self._buffer.context.linked_array_type.from_array(
-            kene0, mode='readonly',
-            container=self)
+        out  = self._buffer.context.linked_array_type.from_array(
+            kene0,
+            mode='setitem_from_container',
+            container=self,
+            container_setitem_name='_kinetic_energy0_setitem')
+        return out
+
+    @kinetic_energy0.setter
+    def kinetic_energy0(self, value):
+        self.kinetic_energy0[:] = value
 
     @property
     def energy(self):
@@ -1483,13 +1542,13 @@ class Particles(xo.HybridClass):
         src_lines.append('if (set_scalar){')
         for _, vv in cls.size_vars + cls.scalar_vars:
             src_lines.append(
-                f'  ParticlesData_set_' + vv + '(dest,'
+                '  ParticlesData_set_' + vv + '(dest,'
                                                f'      LocalParticle_get_{vv}(source));')
         src_lines.append('}')
 
         for _, vv in cls.per_particle_vars:
             src_lines.append(
-                f'  ParticlesData_set_' + vv + '(dest, id, '
+                '  ParticlesData_set_' + vv + '(dest, id, '
                                                f'      LocalParticle_get_{vv}(source));')
         src_lines.append('}')
         src_local_to_particles = '\n'.join(src_lines)
@@ -1539,7 +1598,7 @@ class Particles(xo.HybridClass):
         for tt, vv in cls.size_vars + cls.scalar_vars:
             src_lines.append('/*gpufun*/')
             src_lines.append(f'{tt._c_type} LocalParticle_get_' + vv
-                             + f'(LocalParticle* part)'
+                             + '(LocalParticle* part)'
                              + '{')
             src_lines.append(f'  return part->{vv};')
             src_lines.append('}')
@@ -1547,7 +1606,7 @@ class Particles(xo.HybridClass):
         for tt, vv in cls.per_particle_vars:
             src_lines.append('/*gpufun*/')
             src_lines.append(f'{tt._c_type} LocalParticle_get_' + vv
-                             + f'(LocalParticle* part)'
+                             + '(LocalParticle* part)'
                              + '{')
             src_lines.append(f'  return part->{vv}[part->ipart];')
             src_lines.append('}')
@@ -1565,12 +1624,12 @@ class Particles(xo.HybridClass):
                 src_angles_lines.append(f'    double const p{xx} = LocalParticle_get_p{xx}(part);')
                 if exact == 'exact_':
                     src_angles_lines.append(f'    double const p{yy} = LocalParticle_get_p{yy}(part);')
-                    src_angles_lines.append(f'    double const one_plus_delta = 1. + LocalParticle_get_delta(part);')
+                    src_angles_lines.append('    double const one_plus_delta = 1. + LocalParticle_get_delta(part);')
                     src_angles_lines.append(
-                        f'    double const rpp = 1./sqrt(one_plus_delta*one_plus_delta - px*px - py*py);')
+                        '    double const rpp = 1./sqrt(one_plus_delta*one_plus_delta - px*px - py*py);')
                 else:
-                    src_angles_lines.append(f'    double const rpp = LocalParticle_get_rpp(part);')
-                src_angles_lines.append(f'    // INFO: this is not the angle, but sin(angle)')
+                    src_angles_lines.append('    double const rpp = LocalParticle_get_rpp(part);')
+                src_angles_lines.append('    // INFO: this is not the angle, but sin(angle)')
                 src_angles_lines.append(f'    return p{xx}*rpp;')
                 src_angles_lines.append('}')
                 src_angles_lines.append('')
@@ -1580,15 +1639,15 @@ class Particles(xo.HybridClass):
                 src_angles_lines.append('/*gpufun*/')
                 src_angles_lines.append(f'void LocalParticle_set_{exact}{xx}p(LocalParticle* part, double {xx}p){{')
                 src_angles_lines.append(f'#ifndef FREEZE_VAR_p{xx}')
-                src_angles_lines.append(f'    double rpp = LocalParticle_get_rpp(part);')
+                src_angles_lines.append('    double rpp = LocalParticle_get_rpp(part);')
                 if exact == 'exact_':
                     src_angles_lines.append(
                         f'    // Careful! If {yy}p also changes, use LocalParticle_set_{exact}xp_yp!')
                     src_angles_lines.append(f'    double const {yy}p = LocalParticle_get_{exact}{yy}p(part);')
-                    src_angles_lines.append(f'    rpp *= sqrt(1 + xp*xp + yp*yp);')
+                    src_angles_lines.append('    rpp *= sqrt(1 + xp*xp + yp*yp);')
                 src_angles_lines.append(f'    // INFO: {xx}p is not the angle, but sin(angle)')
                 src_angles_lines.append(f'    LocalParticle_set_p{xx}(part, {xx}p/rpp);')
-                src_angles_lines.append(f'#endif')
+                src_angles_lines.append('#endif')
                 src_angles_lines.append('}')
                 src_angles_lines.append('')
 
@@ -1599,7 +1658,7 @@ class Particles(xo.HybridClass):
                 src_angles_lines.append(f'#ifndef FREEZE_VAR_p{xx}')
                 src_angles_lines.append(f'    LocalParticle_set_{exact}{xx}p(part, '
                                         + f'LocalParticle_get_{exact}{xx}p(part) + {xx}p);')
-                src_angles_lines.append(f'#endif')
+                src_angles_lines.append('#endif')
                 src_angles_lines.append('}')
                 src_angles_lines.append('')
                 # Scaler
@@ -1608,19 +1667,19 @@ class Particles(xo.HybridClass):
                 src_angles_lines.append(f'#ifndef FREEZE_VAR_p{xx}')
                 src_angles_lines.append(f'    LocalParticle_set_{exact}{xx}p(part, '
                                         + f'LocalParticle_get_{exact}{xx}p(part) * value);')
-                src_angles_lines.append(f'#endif')
+                src_angles_lines.append('#endif')
                 src_angles_lines.append('}')
                 src_angles_lines.append('')
             # Double setter, adder, scaler
             src_angles_lines.append('/*gpufun*/')
             src_angles_lines.append(f'void LocalParticle_set_{exact}xp_yp(LocalParticle* part, double xp, double yp){{')
-            src_angles_lines.append(f'    double rpp = LocalParticle_get_rpp(part);')
+            src_angles_lines.append('    double rpp = LocalParticle_get_rpp(part);')
             if exact == 'exact_':
-                src_angles_lines.append(f'    rpp *= sqrt(1 + xp*xp + yp*yp);')
+                src_angles_lines.append('    rpp *= sqrt(1 + xp*xp + yp*yp);')
             for xx in ['x', 'y']:
                 src_angles_lines.append(f'#ifndef FREEZE_VAR_p{xx}')
                 src_angles_lines.append(f'    LocalParticle_set_p{xx}(part, {xx}p/rpp);')
-                src_angles_lines.append(f'#endif')
+                src_angles_lines.append('#endif')
             src_angles_lines.append('}')
             src_angles_lines.append('')
             src_angles_lines.append('/*gpufun*/')
@@ -1866,6 +1925,11 @@ class Particles(xo.HybridClass):
 
                     /*gpufun*/
                     void global_aperture_check(LocalParticle* part0) {
+                        if (LocalParticle_check_track_flag(
+                            part0, XS_FLAG_IGNORE_GLOBAL_APERTURE)){
+                            return;
+                        }
+
                         //start_per_particle_block (part0->part)
                             double const x = LocalParticle_get_x(part);
                             double const y = LocalParticle_get_y(part);
@@ -1984,15 +2048,8 @@ class Particles(xo.HybridClass):
 
     @classmethod
     def reference_from_pdg_id(cls, pdg_id, **kwargs):
-        pdg_id = get_pdg_id_from_name(pdg_id)
-        kwargs['pdg_id'] = pdg_id
-        q0 = kwargs.get('q0')
-        mass0 = kwargs.get('mass0')
-        if q0 is None:
-            q, _, _, _ = get_properties_from_pdg_id(pdg_id)
-            kwargs['q0'] = q
-        if mass0 is None:
-            kwargs['mass0'] = get_mass_from_pdg_id(pdg_id)
+
+        _update_kwargs0_from_pdg_id(pdg_id, kwargs)
 
         particle_ref = cls(**kwargs)
         if particle_ref._capacity > 1:
@@ -2056,8 +2113,10 @@ class Particles(xo.HybridClass):
         getattr(self, varname)[mask] = target_val[mask]
 
     def _update_refs(self, p0c=None, energy0=None, gamma0=None, beta0=None,
+                     kinetic_energy0=None, rigidity0=None,
                      mask=None):
-        if not any(ff is not None for ff in (p0c, energy0, gamma0, beta0)):
+        if not any(ff is not None for ff in (p0c, energy0, gamma0, beta0, 
+                                            kinetic_energy0, rigidity0)):
             self._p0c = 1e9
             p0c = self._p0c
 
@@ -2082,6 +2141,16 @@ class Particles(xo.HybridClass):
             _energy0 = self.mass0 * _gamma0
             _p0c = _energy0 * beta0
             _beta0 = beta0
+        elif kinetic_energy0 is not None:
+            _energy0 = kinetic_energy0 + self.mass0
+            _p0c = _sqrt(_energy0 ** 2 - self.mass0 ** 2)
+            _beta0 = _p0c / _energy0
+            _gamma0 = _energy0 / self.mass0
+        elif rigidity0 is not None:
+            _p0c = rigidity0 * abs(self.q0) * clight
+            _energy0 = _sqrt(_p0c ** 2 + self.mass0 ** 2)
+            _beta0 = _p0c / _energy0
+            _gamma0 = _energy0 / self.mass0
         else:
             raise RuntimeError('This statement is unreachable.')
 
@@ -2252,3 +2321,14 @@ def _mask_to_where(mask, ctx):
 
 def reference_from_pdg_id(pdg_id, **kwargs):
     return Particles.reference_from_pdg_id(pdg_id, **kwargs)
+
+def _update_kwargs0_from_pdg_id(pdg_id, kwargs):
+        pdg_id = get_pdg_id_from_name(pdg_id)
+        kwargs['pdg_id'] = pdg_id
+        q0 = kwargs.get('q0')
+        mass0 = kwargs.get('mass0')
+        if q0 is None:
+            q, _, _, _ = get_properties_from_pdg_id(pdg_id)
+            kwargs['q0'] = q
+        if mass0 is None:
+            kwargs['mass0'] = get_mass_from_pdg_id(pdg_id)
\ No newline at end of file
