@@ -23,12 +23,12 @@ class PipelineCommunicator:
         self.messages[dest][tag].append(send_buffer)
         return self
 
-    def Recv(self,recieve_buffer,source,tag):
+    def Recv(self,receive_buffer,source,tag):
         assert source in self.messages.keys()
         assert tag in self.messages[source].keys()
         assert bool(self.messages[source][tag])
         message = self.messages[source][tag].pop(0)
-        recieve_buffer[:] = message[:]
+        receive_buffer[:] = message[:]
 
     def Iprobe(self,source, tag):
         if source in self.messages.keys():
